import nomad.api.exceptions

from nomad.api.base import Requester


class Jobs(Requester):

    """
    The jobs endpoint is used to query the status of existing
    jobs in Nomad and to register new jobs.
    By default, the agent's local region is used.

    https://www.nomadproject.io/docs/http/jobs.html
    """
    ENDPOINT = "jobs"

    def __init__(self, **kwargs):
        super(Jobs, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):
        try:
            jobs = self.get_jobs()

            for j in jobs:
                if j["ID"] == item:
                    return True
                if j["Name"] == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        jobs = self.get_jobs()
        return len(jobs)

    def __getitem__(self, item):
        try:
            jobs = self.get_jobs()

            for j in jobs:
                if j["ID"] == item:
                    return j
                if j["Name"] == item:
                    return j
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def __iter__(self):
        jobs = self.get_jobs()
        return iter(jobs)

    def get_jobs(self, prefix=None, namespace=None):
        """ Lists all the jobs registered with Nomad.

           https://www.nomadproject.io/docs/http/jobs.html
            arguments:
              - prefix :(str) optional, specifies a string to filter jobs on based on an prefix.
                        This is specified as a querystring parameter.
              - namespace :(str) optional, specifies the target namespace. Specifying * would return all jobs.
                        This is specified as a querystring parameter.
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"prefix": prefix}
        if namespace:
            params["namespace"] = namespace

        return self.request(method="get", params=params).json()

    def register_job(self, job):
        """ Register a job with Nomad.

           https://www.nomadproject.io/docs/http/jobs.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(json=job, method="post").json()

    def parse(self, hcl, canonicalize=False):
        """ Parse a HCL Job file. Returns a dict with the JSON formatted job.
            This API endpoint is only supported from Nomad version 0.8.3.

            https://www.nomadproject.io/api/jobs.html#parse-job

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("parse", json={"JobHCL": hcl, "Canonicalize": canonicalize}, method="post", allow_redirects=True).json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Allocation(Requester):
    """
    The allocation endpoint is used to query the a specific allocation.
    By default, the agent's local region is used; another region can be
    specified using the ?region= query parameter.

    https://www.nomadproject.io/docs/http/alloc.html
    """

    ENDPOINT = "allocation"

    def __init__(self, **kwargs):
        super(Allocation, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __contains__(self, item):
        try:
            response = self.get_allocation(item)

            if response["ID"] == item:
                return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):
        try:
            response = self.get_allocation(item)

            if response["ID"] == item:
                return response
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_allocation(self, id):
        """ Query a specific allocation.

           https://www.nomadproject.io/docs/http/alloc.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="get").json()

    def stop_allocation(self, id):
        """ Stop a specific allocation.

           https://www.nomadproject.io/api-docs/allocations/#stop-allocation

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "stop", method="post").json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Deployment(Requester):

    """
    The /deployment endpoints are used to query for and interact with deployments.

    https://www.nomadproject.io/docs/http/deployments.html
    """
    ENDPOINT = "deployment"

    def __init__(self, **kwargs):
        super(Deployment, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):

        try:
            d = self.get_deployment(item)
            return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):

        try:
            d = self.get_deployment(item)

            if d["ID"] == item:
                return d
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_deployment(self, id):
        """ This endpoint reads information about a specific deployment by ID.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="get").json()

    def get_deployment_allocations(self, id):
        """ This endpoint lists the allocations created or modified for the given deployment.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
            returns: list of dicts
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("allocations", id, method="get").json()

    def fail_deployment(self, id):
        """ This endpoint is used to mark a deployment as failed. This should be done to force the scheduler to stop
            creating allocations as part of the deployment or to cause a rollback to a previous job version.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        fail_json = {"DeploymentID": id}
        return self.request("fail", id, json=fail_json, method="post").json()

    def pause_deployment(self, id, pause):
        """ This endpoint is used to pause or unpause a deployment.
            This is done to pause a rolling upgrade or resume it.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
              - pause, Specifies whether to pause or resume the deployment.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        pause_json = {"Pause": pause,
                      "DeploymentID": id}
        return self.request("pause", id, json=pause_json, method="post").json()

    def promote_deployment_all(self, id, all=True):
        """ This endpoint is used to promote task groups that have canaries for a deployment. This should be done when
            the placed canaries are healthy and the rolling upgrade of the remaining allocations should begin.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
              - all, Specifies whether all task groups should be promoted.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        promote_all_json = {"All": all,
                            "DeploymentID": id}
        return self.request("promote", id, json=promote_all_json, method="post").json()

    def promote_deployment_groups(self, id, groups=list()):
        """ This endpoint is used to promote task groups that have canaries for a deployment. This should be done when
            the placed canaries are healthy and the rolling upgrade of the remaining allocations should begin.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
              - groups, (list) Specifies a particular set of task groups that should be promoted
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        promote_groups_json = {"Groups": groups,
                               "DeploymentID": id}
        return self.request("promote", id, json=promote_groups_json, method="post").json()

    def deployment_allocation_health(self, id, healthy_allocations=list(), unhealthy_allocations=list()):
        """ This endpoint is used to set the health of an allocation that is in the deployment manually. In some use
            cases, automatic detection of allocation health may not be desired. As such those task groups can be marked
            with an upgrade policy that uses health_check = "manual". Those allocations must have their health marked
            manually using this endpoint. Marking an allocation as healthy will allow the rolling upgrade to proceed.
            Marking it as failed will cause the deployment to fail.

           https://www.nomadproject.io/docs/http/deployments.html

            arguments:
              - id
              - healthy_allocations, Specifies the set of allocation that should be marked as healthy.
              - unhealthy_allocations,  Specifies the set of allocation that should be marked as unhealthy.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        allocations = {"HealthyAllocationIDs": healthy_allocations,
                       "UnHealthyAllocationIDs": unhealthy_allocations,
                       "DeploymentID": id}
        return self.request("allocation-health", id, json=allocations, method="post").json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Namespace(Requester):

    """
    The job endpoint is used for CRUD on a single namespace.
    By default, the agent's local region is used.

    https://www.nomadproject.io/api/namespaces.html
    """
    ENDPOINT = "namespace"

    def __init__(self, **kwargs):
        super(Namespace, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):

        try:
            j = self.get_namespace(item)
            return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):

        try:
            j = self.get_namespace(item)

            if j["ID"] == item:
                return j
            if j["Name"] == item:
                return j
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_namespace(self, id):
        """ Query a single namespace.

           https://www.nomadproject.io/api/namespaces.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="get").json()

    def create_namespace(self, namespace):
        """ create namespace

           https://www.nomadproject.io/api/namespaces.html

            arguments:
              - id
              - namespace (dict)
            returns: requests.Response
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(json=namespace, method="post")

    def update_namespace(self, id, namespace):
        """ Update namespace

           https://www.nomadproject.io/api/namespaces.html

            arguments:
              - id
              - namespace (dict)
            returns: requests.Response
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, json=namespace, method="post")

    def delete_namespace(self, id):
        """ delete namespace.

           https://www.nomadproject.io/api/namespaces.html

            arguments:
              - id
            returns: requests.Response
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="delete")



from nomad.api.base import Requester


class Operator(Requester):

    """
    The Operator endpoint provides cluster-level tools for
    Nomad operators, such as interacting with the Raft subsystem.

    https://www.nomadproject.io/docs/http/operator.html
    """

    ENDPOINT = "operator"

    def __init__(self, **kwargs):
        super(Operator, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def get_configuration(self, stale=False):
        """ Query the status of a client node registered with Nomad.

            https://www.nomadproject.io/docs/http/operator.html

            returns: dict
            optional arguments:
              - stale, (defaults to False), Specifies if the cluster should respond without an active leader.
                                            This is specified as a querystring parameter.
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """

        params = {"stale": stale}
        return self.request("raft", "configuration", params=params, method="get").json()

    def delete_peer(self, peer_address, stale=False):
        """ Remove the Nomad server with given address from the Raft configuration.
            The return code signifies success or failure.

            https://www.nomadproject.io/docs/http/operator.html

            arguments:
              - peer_address, The address specifies the server to remove and is given as an IP:port
            optional arguments:
              - stale, (defaults to False), Specifies if the cluster should respond without an active leader.
                                            This is specified as a querystring parameter.
            returns: Boolean
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """

        params = {"address": peer_address, "stale": stale}
        return self.request("raft", "peer", params=params, method="delete").ok



from nomad.api.base import Requester


class System(Requester):

    """
    The system endpoint is used to for system maintenance
    and should not be necessary for most users.
    By default, the agent's local region is used.

    https://www.nomadproject.io/docs/http/system.html
    """

    ENDPOINT = "system"

    def __init__(self, **kwargs):
        super(System, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def initiate_garbage_collection(self):
        """ Initiate garbage collection of jobs, evals, allocations and nodes.

            https://www.nomadproject.io/docs/http/system.html

            returns: Boolean
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("gc", method="put").ok

    def reconcile_summaries(self):
        """ This endpoint reconciles the summaries of all registered jobs.

            https://www.nomadproject.io/docs/http/system.html

            returns: Boolean
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("reconcile", "summaries", method="put").ok



import nomad.api.exceptions

from nomad.api.base import Requester


class Node(Requester):

    """
    The node endpoint is used to query the a specific client node.
    By default, the agent's local region is used.

    https://www.nomadproject.io/docs/http/node.html
    """
    ENDPOINT = "node"

    def __init__(self, **kwargs):
        super(Node, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):

        try:
            n = self.get_node(item)
            return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):

        try:
            n = self.get_node(item)

            if n["ID"] == item:
                return n
            if n["Name"] == item:
                return n
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_node(self, id):
        """ Query the status of a client node registered with Nomad.

           https://www.nomadproject.io/docs/http/node.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="get").json()

    def get_allocations(self, id):
        """ Query the allocations belonging to a single node.

           https://www.nomadproject.io/docs/http/node.html

            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "allocations", method="get").json()

    def evaluate_node(self, id):
        """ Creates a new evaluation for the given node.
            This can be used to force run the 
            scheduling logic if necessary.

           https://www.nomadproject.io/docs/http/node.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "evaluate", method="post").json()

    def drain_node(self, id, enable=False):
        """ Toggle the drain mode of the node.
            When enabled, no further allocations will be
            assigned and existing allocations will be migrated.

           https://www.nomadproject.io/docs/http/node.html

            arguments:
              - id (str uuid): node id
              - enable (bool): enable node drain or not to enable node drain
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """

        return self.request(id, "drain", params={"enable": enable}, method="post").json()

    def drain_node_with_spec(self, id, drain_spec, mark_eligible=None):
        """ This endpoint toggles the drain mode of the node. When draining is enabled,
            no further allocations will be assigned to this node, and existing allocations
            will be migrated to new nodes.

            If an empty dictionary is given as drain_spec this will disable/toggle the drain.

            https://www.nomadproject.io/docs/http/node.html

            arguments:
              - id (str uuid): node id
              - drain_spec (dict): https://www.nomadproject.io/api/nodes.html#drainspec
              - mark_eligible (bool): https://www.nomadproject.io/api/nodes.html#markeligible
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        payload = {}

        if drain_spec and mark_eligible is not None:
            payload = {
                "NodeID": id,
                "DrainSpec": drain_spec,
                "MarkEligible": mark_eligible
            }
        elif drain_spec and mark_eligible is None:
            payload = {
                "NodeID": id,
                "DrainSpec": drain_spec
            }
        elif not drain_spec and mark_eligible is not None:
            payload = {
                "NodeID": id,
                "DrainSpec": None,
                "MarkEligible": mark_eligible
            }
        elif not drain_spec and mark_eligible is None:
            payload = {
                "NodeID": id,
                "DrainSpec": None,
            }

        return self.request(id, "drain", json=payload, method="post").json()

    def eligible_node(self, id, eligible=None, ineligible=None):
        """ Toggle the eligibility of the node.

           https://www.nomadproject.io/docs/http/node.html

            arguments:
              - id (str uuid): node id
              - eligible (bool): Set to True to mark node eligible
              - ineligible (bool): Set to True to mark node ineligible
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        payload = {}

        if eligible is not None and ineligible is not None:
            raise nomad.api.exceptions.InvalidParameters
        if eligible is None and ineligible is None:
            raise nomad.api.exceptions.InvalidParameters

        if eligible is not None and eligible:
            payload = {"Eligibility": "eligible", "NodeID": id}
        elif eligible is not None and not eligible:
            payload = {"Eligibility": "ineligible", "NodeID": id}
        elif ineligible is not None:
            payload = {"Eligibility": "ineligible", "NodeID": id}
        elif ineligible is not None and not ineligible:
            payload = {"Eligibility": "eligible", "NodeID": id}

        return self.request(id, "eligibility", json=payload, method="post").json()

    def purge_node(self, id):
        """ This endpoint purges a node from the system. Nodes can still join the cluster if they are alive.
        arguments:
          - id (str uuid): node id
        returns: dict
        raises:
          - nomad.api.exceptions.BaseNomadException
          - nomad.api.exceptions.URLNotFoundNomadException
        """

        return self.request(id, "purge", method="post").json()




import nomad.api.exceptions

from nomad.api.base import Requester


class Namespaces(Requester):

    """
    The namespaces from enterprise solution

    https://www.nomadproject.io/docs/enterprise/namespaces/index.html
    """
    ENDPOINT = "namespaces"

    def __init__(self, **kwargs):
        super(Namespaces, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):
        try:
            namespaces = self.get_namespaces()

            for n in namespaces:
                if n["Name"] == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        namespaces = self.get_namespaces()
        return len(namespaces)

    def __getitem__(self, item):
        try:
            namespaces = self.get_namespaces()

            for n in namespaces:
                if n["Name"] == item:
                    return n
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def __iter__(self):
        namespaces = self.get_namespaces()
        return iter(namespaces)

    def get_namespaces(self, prefix=None):
        """ Lists all the namespaces registered with Nomad.

           https://www.nomadproject.io/docs/enterprise/namespaces/index.html
            arguments:
              - prefix :(str) optional, specifies a string to filter namespaces on based on an prefix.
                        This is specified as a querystring parameter.
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"prefix": prefix}
        return self.request(method="get", params=params).json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Acl(Requester):
    """
    The endpoint manage security ACL and tokens

    https://www.nomadproject.io/api/acl-tokens.html
    """

    ENDPOINT = "acl"

    def __init__(self, **kwargs):
        super(Acl, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def generate_bootstrap(self):
        """ Activate bootstrap token.

            https://www.nomadproject.io/api/acl-tokens.html

            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("bootstrap", method="post").json()

    def get_tokens(self):
        """ Get a list of tokens.

            https://www.nomadproject.io/api/acl-tokens.html

            returns: list

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """

        return self.request("tokens", method="get").json()

    def get_token(self, id):
        """ Retrieve specific token.

            https://www.nomadproject.io/api/acl-tokens.html

            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("token", id, method="get").json()

    def get_self_token(self):
        """ Retrieve self token used for auth.

            https://www.nomadproject.io/api/acl-tokens.html

            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("token", "self", method="get").json()

    def create_token(self, token):
        """ Create token.

            https://www.nomadproject.io/api/acl-tokens.html

            arguments:
                token
            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("token", json=token, method="post").json()

    def delete_token(self, id):
        """ Delete specific token.

            https://www.nomadproject.io/api/acl-tokens.html

            returns: Boolean

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("token", id, method="delete").ok

    def update_token(self, id, token):
        """ Update token.

            https://www.nomadproject.io/api/acl-tokens.html

            arguments:
                - AccdesorID
                - token
            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("token", id, json=token, method="post").json()

    def get_policies(self):
        """ Get a list of policies.

            https://www.nomadproject.io/api/acl-policies.html

            returns: list

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policies", method="get").json()

    def create_policy(self, id, policy):
        """ Create policy.

            https://www.nomadproject.io/api/acl-policies.html

            arguments:
                - policy
            returns: request.Response

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, json=policy, method="post")

    def get_policy(self, id):
        """ Get a spacific.

            https://www.nomadproject.io/api/acl-policies.html

            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, method="get").json()

    def update_policy(self, id, policy):
        """ Create policy.

            https://www.nomadproject.io/api/acl-policies.html

            arguments:
                - name
                - policy
            returns: request.Response

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, json=policy, method="post")

    def delete_policy(self, id):
        """ Delete specific policy.

            https://www.nomadproject.io/api/acl-policies.html

            arguments:
                - id
            returns: Boolean

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, method="delete").ok



import nomad.api.exceptions

from nomad.api.base import Requester


class Evaluation(Requester):

    """
    The evaluation endpoint is used to query a specific evaluations.
    By default, the agent's local region is used; another region can
    be specified using the ?region= query parameter.

    https://www.nomadproject.io/docs/http/eval.html
    """
    ENDPOINT = "evaluation"

    def __init__(self, **kwargs):
        super(Evaluation, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):

        try:
            e = self.get_evaluation(item)
            return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):

        try:
            e = self.get_evaluation(item)

            if e["ID"] == item:
                return e
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_evaluation(self, id):
        """ Query a specific evaluation.

           https://www.nomadproject.io/docs/http/eval.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, method="get").json()

    def get_allocations(self, id):
        """ Query the allocations created or modified by an evaluation.

           https://www.nomadproject.io/docs/http/eval.html

            arguments:
              - id
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "allocations", method="get").json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Status(object):

    """
    By default, the agent's local region is used

    https://www.nomadproject.io/docs/http/status.html
    """

    def __init__(self, **kwargs):
        self.leader = Leader(**kwargs)
        self.peers = Peers(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError


class Leader(Requester):

    ENDPOINT = "status/leader"

    def __contains__(self, item):
        try:
            leader = self.get_leader()

            if leader == item:
                return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        leader = self.get_leader()
        return len(leader)

    def get_leader(self):
        """ Returns the address of the current leader in the region.

            https://www.nomadproject.io/docs/http/status.html

            returns: string
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(method="get").json()


class Peers(Requester):

    ENDPOINT = "status/peers"

    def __contains__(self, item):
        try:
            peers = self.get_peers()

            for p in peers:
                if p == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        peers = self.get_peers()
        return len(peers)

    def __getitem__(self, item):
        try:
            peers = self.get_peers()

            for p in peers:
                if p == item:
                    return p
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def __iter__(self):
        peers = self.get_peers()
        return iter(peers)

    def get_peers(self):
        """ Returns the set of raft peers in the region.

            https://www.nomadproject.io/docs/http/status.html

            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(method="get").json()



import requests
import nomad.api.exceptions

from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class Requester(object):

    ENDPOINT = ""

    def __init__(self, address=None, uri='http://127.0.0.1', port=4646, namespace=None, token=None, timeout=5, version='v1', verify=False, cert=(), region=None, **kwargs):
        self.uri = uri
        self.port = port
        self.namespace = namespace
        self.token = token
        self.timeout = timeout
        self.version = version
        self.verify = verify
        self.cert = cert
        self.address = address
        self.session = requests.Session()
        self.region = region

    def _endpoint_builder(self, *args):
        if args:
            u = "/".join(args)
            return "{v}/".format(v=self.version) + u

    def _required_namespace(self, endpoint):
        required_namespace = [
                                "job",
                                "jobs",
                                "allocation",
                                "allocations",
                                "deployment",
                                "deployments",
                                "acl",
                                "client",
                                "node"
                             ]
        # split 0 -> Api Version
        # split 1 -> Working Endpoint
        ENDPOINT_NAME = 1
        endpoint_split = endpoint.split("/")
        try:
            required = endpoint_split[ENDPOINT_NAME] in required_namespace
        except:
            required = False

        return required

    def _url_builder(self, endpoint):
        url = self.address

        if self.address is None:
            url = "{uri}:{port}".format(uri=self.uri, port=self.port)

        url = "{url}/{endpoint}".format(url=url, endpoint=endpoint)

        return url

    def _query_string_builder(self, endpoint, params=None):
        qs = {}

        if not isinstance(params, dict):
            params = {}

        if ("namespace" not in params) and (self.namespace and self._required_namespace(endpoint)):
            qs["namespace"] = self.namespace

        if "region" not in params and self.region:
            qs["region"] = self.region

        return qs

    def request(self, *args, **kwargs):
        endpoint = self._endpoint_builder(self.ENDPOINT, *args)
        response = self._request(
            endpoint=endpoint,
            method=kwargs.get("method"),
            params=kwargs.get("params", None),
            data=kwargs.get("data", None),
            json=kwargs.get("json", None),
            headers=kwargs.get("headers", None),
            allow_redirects=kwargs.get("allow_redirects", False),
            timeout=kwargs.get("timeout", self.timeout),
            stream=kwargs.get("stream", False)
        )

        return response

    def _request(self, method, endpoint, params=None, data=None, json=None, headers=None, allow_redirects=None, timeout=None, stream=False):
        url = self._url_builder(endpoint)
        qs = self._query_string_builder(endpoint=endpoint, params=params)

        if params:
            params.update(qs)
        else:
            params = qs

        if self.token:
            try:
                headers["X-Nomad-Token"] = self.token
            except TypeError:
                headers = {"X-Nomad-Token": self.token}

        response = None

        try:
            method = method.lower()
            if method == "get":
                response = self.session.get(
                    allow_redirects=allow_redirects,
                    cert=self.cert,
                    headers=headers,
                    params=params,
                    stream=stream,
                    timeout=timeout,
                    url=url,
                    verify=self.verify,
                )

            elif method == "post":
                response = self.session.post(
                    allow_redirects=allow_redirects,
                    cert=self.cert,
                    data=data,
                    headers=headers,
                    json=json,
                    params=params,
                    timeout=timeout,
                    url=url,
                    verify=self.verify,
                )
            elif method == "put":
                response = self.session.put(
                    cert=self.cert,
                    data=data,
                    headers=headers,
                    json=json,
                    params=params,
                    timeout=timeout,
                    url=url,
                    verify=self.verify,
                )
            elif method == "delete":
                response = self.session.delete(
                    cert=self.cert,
                    headers=headers,
                    params=params,
                    timeout=timeout,
                    url=url,
                    verify=self.verify,
                )

            if response.ok:
                return response
            elif response.status_code == 400:
                raise nomad.api.exceptions.BadRequestNomadException(response)
            elif response.status_code == 403:
                raise nomad.api.exceptions.URLNotAuthorizedNomadException(response)
            elif response.status_code == 404:
                raise nomad.api.exceptions.URLNotFoundNomadException(response)
            else:
                raise nomad.api.exceptions.BaseNomadException(response)

        except requests.exceptions.ConnectionError as error:
            if all([stream, timeout]):
                raise nomad.api.exceptions.TimeoutNomadException(error)

            raise nomad.api.exceptions.BaseNomadException(error)

        except requests.RequestException as error:
            raise nomad.api.exceptions.BaseNomadException(error)



from nomad.api.base import Requester


class Allocations(Requester):

    """
    The allocations endpoint is used to query the status of allocations.
    By default, the agent's local region is used; another region can be
    specified using the ?region= query parameter.

    https://www.nomadproject.io/docs/http/allocs.html
    """
    ENDPOINT = "allocations"

    def __init__(self, **kwargs):
        super(Allocations, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __len__(self):
        response = self.get_allocations()
        return len(response)

    def __iter__(self):
        response = self.get_allocations()
        return iter(response)

    def get_allocations(self, prefix=None):
        """ Lists all the allocations.

           https://www.nomadproject.io/docs/http/allocs.html
            arguments:
              - prefix :(str) optional, specifies a string to filter allocations on based on an prefix.
                        This is specified as a querystring parameter.
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"prefix": prefix}
        return self.request(method="get", params=params).json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Job(Requester):

    """
    The job endpoint is used for CRUD on a single job.
    By default, the agent's local region is used.

    https://www.nomadproject.io/docs/http/job.html
    """
    ENDPOINT = "job"

    def __init__(self, **kwargs):
        super(Job, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def __contains__(self, item):

        try:
            j = self.get_job(item)
            return True
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):

        try:
            j = self.get_job(item)

            if j["ID"] == item:
                return j
            if j["Name"] == item:
                return j
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_job(self, id, namespace=None):
        """ Query a single job for its specification and status.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - namespace :(str) optional, specifies the target namespace. Specifying * would return all jobs.
                        This is specified as a querystring parameter.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {}

        if namespace:
            params["namespace"] = namespace

        return self.request(id, method="get", params=params).json()

    def get_versions(self, id):
        """ This endpoint reads information about all versions of a job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: list of dicts
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "versions", method="get").json()

    def get_allocations(self, id):
        """ Query the allocations belonging to a single job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "allocations", method="get").json()

    def get_evaluations(self, id):
        """ Query the evaluations belonging to a single job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "evaluations", method="get").json()

    def get_deployments(self, id):
        """ This endpoint lists a single job's deployments

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "deployments", method="get").json()

    def get_deployment(self, id):
        """ This endpoint returns a single job's most recent deployment.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: list of dicts
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "deployment", method="get").json()

    def get_summary(self, id):
        """ Query the summary of a job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "summary", method="get").json()

    def register_job(self, id, job):
        """ Registers a new job or updates an existing job

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, json=job, method="post").json()

    def evaluate_job(self, id):
        """ Creates a new evaluation for the given job.
            This can be used to force run the scheduling logic if necessary.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "evaluate", method="post").json()

    def plan_job(self, id, job, diff=False, policy_override=False):
        """ Invoke a dry-run of the scheduler for the job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - job, dict
              - diff, boolean
              - policy_override, boolean
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        json_dict = {}
        json_dict.update(job)
        json_dict.setdefault('Diff', diff)
        json_dict.setdefault('PolicyOverride', policy_override)
        return self.request(id, "plan", json=json_dict, method="post").json()

    def periodic_job(self, id):
        """ Forces a new instance of the periodic job. A new instance will be
            created even if it violates the job's prohibit_overlap settings.
            As such, this should be only used to immediately
            run a periodic job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "periodic", "force", method="post").json()

    def dispatch_job(self, id, payload=None, meta=None):
        """ Dispatches a new instance of a parameterized job.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - payload
              - meta
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        dispatch_json = {"Meta": meta, "Payload": payload}
        return self.request(id, "dispatch", json=dispatch_json, method="post").json()

    def revert_job(self, id, version, enforce_prior_version=None):
        """ This endpoint reverts the job to an older version.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - version, Specifies the job version to revert to.
            optional_arguments:
              - enforce_prior_version, Optional value specifying the current job's version.
                                       This is checked and acts as a check-and-set value before reverting to the
                                       specified job.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        revert_json = {"JobID": id,
                       "JobVersion": version,
                       "EnforcePriorVersion": enforce_prior_version}
        return self.request(id, "revert", json=revert_json, method="post").json()

    def stable_job(self, id, version, stable):
        """ This endpoint sets the job's stability.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - version, Specifies the job version to revert to.
              - stable, Specifies whether the job should be marked as stable or not.
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        revert_json = {"JobID": id,
                       "JobVersion": version,
                       "Stable": stable}
        return self.request(id, "stable", json=revert_json, method="post").json()

    def deregister_job(self, id, purge=None):
        """ Deregisters a job, and stops all allocations part of it.

           https://www.nomadproject.io/docs/http/job.html

            arguments:
              - id
              - purge (bool), optionally specifies whether the job should be
                stopped and purged immediately (`purge=True`) or deferred to the
                Nomad garbage collector (`purge=False`).

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
              - nomad.api.exceptions.InvalidParameters
        """
        params = None
        if purge is not None:
            if not isinstance(purge, bool):
                raise nomad.api.exceptions.InvalidParameters("purge is invalid "
                        "(expected type %s but got %s)"%(type(bool()), type(purge)))
            params = {"purge": purge}
        return self.request(id, params=params, method="delete").json()



import nomad.api.exceptions

from nomad.api.base import Requester


class Agent(Requester):

    """The self endpoint is used to query the state of the target agent."""
    ENDPOINT = "agent"

    def __init__(self, **kwargs):
        super(Agent, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        msg = "{0} does not exist".format(item)
        raise AttributeError(msg)

    def get_agent(self):
        """ Query the state of the target agent.

            https://www.nomadproject.io/docs/http/agent-self.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("self", method="get").json()

    def get_members(self):
        """Lists the known members of the gossip pool.

           https://www.nomadproject.io/docs/http/agent-members.html

            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("members", method="get").json()

    def get_servers(self):
        """ Lists the known members of the gossip pool.

            https://www.nomadproject.io/docs/http/agent-servers.html

            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("servers", method="get").json()

    def join_agent(self, addresses):
        """Initiate a join between the agent and target peers.

           https://www.nomadproject.io/docs/http/agent-join.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"address": addresses}
        return self.request("join", params=params, method="post").json()

    def update_servers(self, addresses):
        """Updates the list of known servers to the provided list.
           Replaces all previous server addresses with the new list.

           https://www.nomadproject.io/docs/http/agent-servers.html

            returns: 200 status code
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"address": addresses}
        return self.request("servers", params=params, method="post").status_code

    def force_leave(self, node):
        """Force a failed gossip member into the left state.

            https://www.nomadproject.io/docs/http/agent-force-leave.html

            returns: 200 status code
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"node": node}
        return self.request("force-leave", params=params, method="post").status_code



from nomad.api.base import Requester


class Client(object):

    def __init__(self, **kwargs):
        self.ls = ls(**kwargs)
        self.cat = cat(**kwargs)
        self.stat = stat(**kwargs)
        self.stats = stats(**kwargs)
        self.allocation = allocation(**kwargs)
        self.read_at = read_at(**kwargs)
        self.stream_file = stream_file(**kwargs)
        self.stream_logs = stream_logs(**kwargs)
        self.gc_allocation = gc_allocation(**kwargs)
        self.gc_all_allocations = gc_all_allocations(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError


class ls(Requester):

    """
    The /fs/ls endpoint is used to list files in an allocation directory.
    This API endpoint is hosted by the Nomad client and requests have to be
    made to the Nomad client where the particular allocation was placed.

    https://www.nomadproject.io/docs/http/client-fs-ls.html
    """

    ENDPOINT = "client/fs/ls"

    def __init__(self, **kwargs):
        super(ls, self).__init__(**kwargs)

    def list_files(self, id=None, path="/"):
        """ List files in an allocation directory.

           https://www.nomadproject.io/docs/http/client-fs-ls.html

            arguments:
              - id
              - path          
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        if id:
            return self.request(id, params={"path": path}, method="get").json()
        else:
            return self.request(params={"path": path}, method="get").json()


class cat(Requester):

    """
    The /fs/cat endpoint is used to read the contents of a file in an
    allocation directory. This API endpoint is hosted by the Nomad
    client and requests have to be made to the Nomad client where the
    particular allocation was placed.

    https://www.nomadproject.io/docs/http/client-fs-cat.html
    """

    ENDPOINT = "client/fs/cat"

    def __init__(self, **kwargs):
        super(cat, self).__init__(**kwargs)

    def read_file(self, id=None, path="/"):
        """ Read contents of a file in an allocation directory.

           https://www.nomadproject.io/docs/http/client-fs-cat.html

            arguments:
              - id
              - path
            returns: (str) text
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        if id:
            return self.request(id, params={"path": path}, method="get").text
        else:
            return self.request(params={"path": path}, method="get").text


class read_at(Requester):

    """
    This endpoint reads the contents of a file in an allocation directory at a particular offset and limit.

    https://www.nomadproject.io/api/client.html#read-file-at-offset
    """

    ENDPOINT = "client/fs/readat"

    def __init__(self, **kwargs):
        super(read_at, self).__init__(**kwargs)

    def read_file_offset(self, id, offset, limit, path="/"):
        """ Read contents of a file in an allocation directory.

           https://www.nomadproject.io/docs/http/client-fs-cat.html

            arguments:
              - id: (str) allocation_id required
              - offset: (int) required
              - limit: (int) required
              - path: (str) optional
            returns: (str) text
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.BadRequestNomadException
        """
        params = {
            "path": path,
            "offset": offset,
            "limit": limit
        }
        return self.request(id, params=params, method="get").text


class stream_file(Requester):

    """
    This endpoint streams the contents of a file in an allocation directory.

    https://www.nomadproject.io/api/client.html#stream-file
    """

    ENDPOINT = "client/fs/stream"

    def __init__(self, **kwargs):
        super(stream_file, self).__init__(**kwargs)

    def stream(self, id, offset, origin, path="/"):
        """ This endpoint streams the contents of a file in an allocation directory.

            https://www.nomadproject.io/api/client.html#stream-file

            arguments:
              - id: (str) allocation_id required
              - offset: (int) required
              - origin: (str) either start|end
              - path: (str) optional
            returns: (str) text
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.BadRequestNomadException
        """
        params = {
            "path": path,
            "offset": offset,
            "origin": origin
        }
        return self.request(id, params=params, method="get").text


class stream_logs(Requester):

    """
    This endpoint streams a task's stderr/stdout logs.

    https://www.nomadproject.io/api/client.html#stream-logs
    """

    ENDPOINT = "client/fs/logs"

    def __init__(self, **kwargs):
        super(stream_logs, self).__init__(**kwargs)

    def stream(self, id, task, type, follow=False, offset=0, origin="start", plain=False):
        """ This endpoint streams a task's stderr/stdout logs.

            https://www.nomadproject.io/api/client.html#stream-logs

            arguments:
              - id: (str) allocation_id required
              - task: (str) name of the task inside the allocation to stream logs from
              - type: (str) Specifies the stream to stream. Either "stderr|stdout"
              - follow: (bool) default false
              - offset: (int) default 0
              - origin: (str) either start|end, default "start"
              - plain: (bool) Return just the plain text without framing. default False
            returns: (str) text
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.BadRequestNomadException
        """
        params = {
            "task": task,
            "type": type,
            "follow": follow,
            "offset": offset,
            "origin": origin,
            "plain": plain
        }
        return self.request(id, params=params, method="get").text


class stat(Requester):

    """
    The /fs/stat endpoint is used to show stat information 
    This API endpoint is hosted by the Nomad client and requests have to be
    made to the Nomad client where the particular allocation was placed.

    https://www.nomadproject.io/docs/http/client-fs-stat.html
    """

    ENDPOINT = "client/fs/stat"

    def __init__(self, **kwargs):
        super(stat, self).__init__(**kwargs)

    def stat_file(self, id=None, path="/"):
        """ Stat a file in an allocation directory.

           https://www.nomadproject.io/docs/http/client-fs-stat.html

            arguments:
              - id
              - path
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        if id:
            return self.request(id, params={"path": path}, method="get").json()
        else:
            return self.request(params={"path": path}, method="get").json()


class stats(Requester):

    """
    The /stats endpoint queries the actual resources consumed on a node.
    The API endpoint is hosted by the Nomad client and requests have to
    be made to the nomad client whose resource usage metrics are of interest.

    https://www.nomadproject.io/api/client.html#read-stats
    """

    ENDPOINT = "client/stats"

    def __init__(self, **kwargs):
        super(stats, self).__init__(**kwargs)

    def read_stats(self, node_id=None):
        """ Query the actual resources consumed on a node.

            https://www.nomadproject.io/api/client.html#read-stats

            arguments:
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(params={"node_id": node_id}, method="get").json()


class allocation(Requester):

    """
    The allocation/:alloc_id/stats endpoint is used to query the actual
    resources consumed by an allocation. The API endpoint is hosted by the 
    Nomad client and requests have to be made to the nomad client whose 
    resource usage metrics are of interest.

    https://www.nomadproject.io/api/client.html#read-allocation
    """

    ENDPOINT = "client/allocation"

    def __init__(self, **kwargs):
        super(allocation, self).__init__(**kwargs)

    def read_allocation_stats(self, id):
        """ Query the actual resources consumed by an allocation.

            https://www.nomadproject.io/api/client.html#read-allocation

            arguments:
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "stats", method="get").json()

    def restart_allocation(self, id):
        """ Restart a specific allocation.

           https://www.nomadproject.io/api-docs/allocations/#restart-allocation

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(id, "restart", method="post").json()


class gc_allocation(Requester):

    """
    This endpoint forces a garbage collection of a particular, stopped allocation on a node.

    https://www.nomadproject.io/api/client.html#gc-allocation
    """

    ENDPOINT = "client/allocation"

    def __init__(self, **kwargs):
        super(gc_allocation, self).__init__(**kwargs)

    def garbage_collect(self, id):
        """ This endpoint forces a garbage collection of a particular, stopped allocation on a node.

            https://www.nomadproject.io/api/client.html#gc-allocation

            arguments:
              - id: (str) full allocation_id
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        self.request(id, "gc", method="get")


class gc_all_allocations(Requester):

    """
    This endpoint forces a garbage collection of all stopped allocations on a node.

    https://www.nomadproject.io/api/client.html#gc-all-allocation
    """

    ENDPOINT = "client/gc"

    def __init__(self, **kwargs):
        super(gc_all_allocations, self).__init__(**kwargs)

    def garbage_collect(self, node_id=None):
        """ This endpoint forces a garbage collection of all stopped allocations on a node.

            https://www.nomadproject.io/api/client.html#gc-all-allocation

            arguments:
              - node_id: (str) full allocation_id
            raises:
              - nomad.api.exceptions.BaseNomadException
        """
        self.request(params={"node_id": node_id}, method="get")



from nomad.api.base import Requester


class Sentinel(Requester):

    """
    The endpoint manage sentinel policies (Enterprise Only)

    https://www.nomadproject.io/api/sentinel-policies.html
    """

    ENDPOINT = "sentinel"

    def __init__(self, **kwargs):
        super(Sentinel, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def get_policies(self):
        """ Get a list of policies.

            https://www.nomadproject.io/api/sentinel-policies.html

            returns: list

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policies", method="get").json()

    def create_policy(self, id, policy):
        """ Create policy.

            https://www.nomadproject.io/api/sentinel-policies.html

            arguments:
                - policy
            returns: requests.Response

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, json=policy, method="post")

    def get_policy(self, id):
        """ Get a spacific policy.

            https://www.nomadproject.io/api/sentinel-policies.html

            returns: dict

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, method="get").json()

    def update_policy(self, id, policy):
        """ Create policy.

            https://www.nomadproject.io/api/sentinel-policies.html

            arguments:
                - name
                - policy
            returns: requests.Response

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, json=policy, method="post")

    def delete_policy(self, id):
        """ Delete specific policy.

            https://www.nomadproject.io/api/sentinel-policies.html

            arguments:
                - id
            returns: Boolean

            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("policy", id, method="delete").ok



import nomad.api.exceptions

from nomad.api.base import Requester


class Regions(Requester):

    """
    https://www.nomadproject.io/docs/http/regions.html
    """
    ENDPOINT = "regions"

    def __init__(self, **kwargs):
        super(Regions, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __contains__(self, item):
        try:
            regions = self.get_regions()

            for r in regions:
                if r == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        regions = self.get_regions()
        return len(regions)

    def __getitem__(self, item):
        try:
            regions = self.get_regions()

            for r in regions:
                if r == item:
                    return r
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def __iter__(self):
        regions = self.get_regions()
        return iter(regions)

    def get_regions(self):
        """ Returns the known region names.

            https://www.nomadproject.io/docs/http/regions.html

            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(method="get").json()



import json
import threading
import queue

import requests

from nomad.api.base import Requester
from nomad.api.exceptions import TimeoutNomadException


class Event(object):

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __init__(self, **kwargs):
        self.stream = stream(**kwargs)


class stream(Requester):
    """
    The /event/stream endpoint is used to stream events generated by Nomad.

    https://www.nomadproject.io/api-docs/events
    """

    ENDPOINT = "event/stream"

    def __init__(self, **kwargs):
        super(stream, self).__init__(**kwargs)

    def _get_stream(self, method, params, timeout, event_queue, exit_event):
        """
        Used as threading target, to obtain json() value
        Args:
            method:
            params:
            timeout:
            event_queue:
            exit_event:
        """

        while exit_event.is_set() is False:
            try:
                with self.request(method=method, params=params, timeout=timeout, stream=True) as resp:
                    for raw_msg in resp.iter_lines():
                        msg = json.loads(raw_msg)

                        # don't send heartbeats
                        if msg:
                            event_queue.put(msg)

                        if exit_event.is_set():
                            return

            except requests.exceptions.ConnectionError:
                continue

    def get_stream(self, index=0, topic=None, namespace=None, event_queue=None, timeout=None):
        """
        Usage:
            stream, stream_exit_event, events = n.event.stream.get_stream()
            stream.start()

            while True:
                event = events.get()
                print(event)
                events.task_done()

        Args:
            index: (int),  Specifies the index to start streaming events from. If the requested index is no longer
                in the buffer the stream will start at the next available index.

            topic: (None or dict), Specifies a topic to subscribe to and filter on. The default is to subscribe to all topics.
                Multiple topics may be specified by passing multiple topic parameters.
                A valid topic parameter includes a topic type and an optional filter_key separated by a colon :.
                As an example ?topic=Deployment:redis would subscribe to all Deployment events for a job redis.
                an additional topic &topic=Deployment:web would include deployment events for redis and web.
                To only subscribe to Node events a topic parameter of ?topic=Node without a separator value
                would be used. ?topic=Node:* is also valid.

            namespace: (str) Specifies the target namespace to filter on. Specifying * includes all namespaces
                for event types that support namespaces.

            event_queue: (None or queue.Queue) for thread listener to push events onto.

            timeout: (None or int), override timeout (seconds) so connection is not closed.
                Defaults to timeout in constructor if not given.

        Returns: (threading.Thread), (threading.Event) (queue.Queue)
        """

        params = {
            "index": index,
        }

        if namespace:
            params["namespace"] = namespace

        if topic:
            params["topic"] = topic

        if event_queue is None:
            event_queue = queue.Queue()

        stream_exit_event = threading.Event()
        _stream = threading.Thread(
            name="python-nomad-event-stream",
            target=self._get_stream,
            kwargs={
                "method": "get",
                "params": params,
                "timeout": timeout,
                "event_queue": event_queue,
                "exit_event": stream_exit_event
            }
        )

        return _stream, stream_exit_event, event_queue



import nomad.api.exceptions

from nomad.api.base import Requester


class Deployments(Requester):

    """
    The /deployment endpoints are used to query for and interact with deployments.

    https://www.nomadproject.io/docs/http/deployments.html
    """
    ENDPOINT = "deployments"

    def __init__(self, **kwargs):
        super(Deployments, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __len__(self):
        response = self.get_deployments()
        return len(response)

    def __iter__(self):
        response = self.get_deployments()
        return iter(response)

    def __contains__(self, item):
        try:
            deployments = self.get_deployments()

            for d in deployments:
                if d["ID"] == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __getitem__(self, item):
        try:
            deployments = self.get_deployments()

            for d in deployments:
                if d["ID"] == item:
                    return d
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def get_deployments(self, prefix=""):
        """ This endpoint lists all deployments.

           https://www.nomadproject.io/docs/http/deployments.html

            optional_arguments:
              - prefix, (default "") Specifies a string to filter deployments on based on an index prefix.
                        This is specified as a querystring parameter.

            returns: list of dicts
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"prefix": prefix}
        return self.request(params=params, method="get").json()



import nomad.api.exceptions

from nomad.api.base import Requester

class Evaluations(Requester):

    """
    The evaluations endpoint is used to query the status of evaluations.
    By default, the agent's local region is used; another region can
    be specified using the ?region= query parameter.

    https://www.nomadproject.io/docs/http/evals.html
    """
    ENDPOINT = "evaluations"

    def __init__(self, **kwargs):
        super(Evaluations, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def __contains__(self, item):
        try:
            evaluations = self.get_evaluations()

            for e in evaluations:
                if e["ID"] == item:
                    return True
            else:
                return False
        except nomad.api.exceptions.URLNotFoundNomadException:
            return False

    def __len__(self):
        evaluations = self.get_evaluations()
        return len(evaluations)

    def __getitem__(self, item):
        try:
            evaluations = self.get_evaluations()

            for e in evaluations:
                if e["ID"] == item:
                    return e
            else:
                raise KeyError
        except nomad.api.exceptions.URLNotFoundNomadException:
            raise KeyError

    def __iter__(self):
        evaluations = self.get_evaluations()
        return iter(evaluations)

    def get_evaluations(self, prefix=None):
        """ Lists all the evaluations.

           https://www.nomadproject.io/docs/http/evals.html
            arguments:
              - prefix :(str) optional, specifies a string to filter evaluations on based on an prefix.
                        This is specified as a querystring parameter.
            returns: list
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        params = {"prefix": prefix}
        return self.request(method="get", params=params).json()



import nomad
import threading
import time
import queue


def stop_stream(exit_event, timeout):
    print("start sleep")
    time.sleep(timeout)
    print("set exit event")
    exit_event.set()


n = nomad.Nomad("0.0.0.0")

stream, stream_exit_event, events = n.event.stream.get_stream(index=0, timeout=30.0)
stream.start()

stop = threading.Thread(target=stop_stream, args=(stream_exit_event, 28.0))
stop.start()

while True:
    if not stream.is_alive():
        print("not alive")
        break

    try:
        event = events.get(timeout=1.0)
        print(event)
        events.task_done()
    except queue.Empty:
        continue










import nomad.api.exceptions
from nomad.api.acl import Acl
from nomad.api.agent import Agent
from nomad.api.allocation import Allocation
from nomad.api.allocations import Allocations
from nomad.api.base import Requester
from nomad.api.client import Client
from nomad.api.deployment import Deployment
from nomad.api.deployments import Deployments
from nomad.api.evaluation import Evaluation
from nomad.api.evaluations import Evaluations
from nomad.api.event import Event
from nomad.api.job import Job
from nomad.api.jobs import Jobs
from nomad.api.metrics import Metrics
from nomad.api.namespace import Namespace
from nomad.api.namespaces import Namespaces
from nomad.api.node import Node
from nomad.api.nodes import Nodes
from nomad.api.operator import Operator
from nomad.api.regions import Regions
from nomad.api.sentinel import Sentinel
from nomad.api.status import Status
from nomad.api.system import System
from nomad.api.validate import Validate



import requests


class BaseNomadException(Exception):
    """General Error occurred when interacting with nomad API"""
    def __init__(self, nomad_resp):
        self.nomad_resp = nomad_resp

    def __str__(self):
        if isinstance(self.nomad_resp, requests.Response) and hasattr(self.nomad_resp, "text"):
            return 'The {0} was raised with following response: {1}.'.format(self.__class__.__name__, self.nomad_resp.text)
        else:
            return 'The {0} was raised due to the following error: {1}'.format(self.__class__.__name__,  str(self.nomad_resp))


class URLNotFoundNomadException(BaseNomadException):
    """The requeted URL given does not exist"""


class URLNotAuthorizedNomadException(BaseNomadException):
    """The requested URL is not authorized. ACL"""


class BadRequestNomadException(BaseNomadException):
    """Validation failure and if a parameter is modified in the request, it could potentially succeed."""


class InvalidParameters(Exception):
    """Invalid parameters given"""


class TimeoutNomadException(requests.exceptions.RequestException):
    """Timeout on request, if using a stream and timeout in conjunction"""



from nomad.api.base import Requester


class Validate(Requester):

    """
    The system endpoint is used to for system maintenance
    and should not be necessary for most users.
    By default, the agent's local region is used.

    https://www.nomadproject.io/docs/http/system.html
    """

    ENDPOINT = "validate"

    def __init__(self, **kwargs):
        super(Validate, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def validate_job(self, nomad_job_dict):
        """ This endpoint validates a Nomad job file. The local Nomad agent forwards the request to a server.
        In the event a server can't be reached the agent verifies the job file locally but skips validating driver
        configurations.

            https://www.nomadproject.io/api/validate.html

            arguments:
              - nomad_job_json, any valid nomad job IN dict FORMAT
            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request("job", json=nomad_job_dict, method="post")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='python-nomad',
    version='1.4.0',
    install_requires=['requests'],
    packages=['nomad', 'nomad.api'],
    url='http://github.com/jrxfive/python-nomad',
    license='MIT',
    author='jrxfive',
    author_email='jrxfive@gmail.com',
    description='Client library for Hashicorp Nomad',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='nomad hashicorp client',
)



import nomad.api as api
import os


class Nomad(object):

    def __init__(self,
                 host='127.0.0.1',
                 secure=False,
                 port=4646,
                 address=os.getenv('NOMAD_ADDR', None),
                 namespace=os.getenv('NOMAD_NAMESPACE', None),
                 token=os.getenv('NOMAD_TOKEN', None),
                 timeout=5,
                 region=os.getenv('NOMAD_REGION', None),
                 version='v1',
                 verify=False,
                 cert=(os.getenv('NOMAD_CLIENT_CERT', None),
                       os.getenv('NOMAD_CLIENT_KEY', None))):
        """ Nomad api client

          https://github.com/jrxFive/python-nomad/

           optional arguments:
            - host (defaults 127.0.0.1), string ip or name of the nomad api server/agent that will be used.
            - port (defaults 4646), integer port that will be used to connect.
            - secure (defaults False), define if the protocol is secured or not (https or http)
            - version (defaults v1), vesion of the api of nomad.
            - verify (defaults False), verify the certificate when tls/ssl is enabled
                                at nomad.
            - cert (defaults empty), cert, or key and cert file to validate the certificate
                                configured at nomad.
            - region (defaults None), version of the region to use. It will be used then
                                regions of the current agent of the connection.
            - namespace (defaults to None), Specifies the enterpise namespace that will
                                be use to deploy or to ask info to nomad.
            - token (defaults to None), Specifies to append ACL token to the headers to
                                make authentication on secured based nomad environemnts.
           returns: Nomad api client object

           raises:
             - nomad.api.exceptions.BaseNomadException
             - nomad.api.exceptions.URLNotFoundNomadException
             - nomad.api.exceptions.URLNotAuthorizedNomadException
        """
        self.host = host
        self.secure = secure
        self.port = port
        self.address = address
        self.region = region
        self.timeout = timeout
        self.version = version
        self.token = token
        self.verify = verify
        self.cert = cert if all(cert) else ()
        self.__namespace = namespace

        self.requester_settings = {
            "address": self.address,
            "uri": self.get_uri(),
            "port": self.port,
            "namespace": self.__namespace,
            "token": self.token,
            "timeout": self.timeout,
            "version": self.version,
            "verify": self.verify,
            "cert": self.cert,
            "region": self.region
        }

        self._acl = api.Acl(**self.requester_settings)
        self._agent = api.Agent(**self.requester_settings)
        self._allocation = api.Allocation(**self.requester_settings)
        self._allocations = api.Allocations(**self.requester_settings)
        self._client = api.Client(**self.requester_settings)
        self._deployment = api.Deployment(**self.requester_settings)
        self._deployments = api.Deployments(**self.requester_settings)
        self._evaluation = api.Evaluation(**self.requester_settings)
        self._evaluations = api.Evaluations(**self.requester_settings)
        self._event = api.Event(**self.requester_settings)
        self._job = api.Job(**self.requester_settings)
        self._jobs = api.Jobs(**self.requester_settings)
        self._metrics = api.Metrics(**self.requester_settings)
        self._namespace = api.Namespace(**self.requester_settings)
        self._namespaces = api.Namespaces(**self.requester_settings)
        self._node = api.Node(**self.requester_settings)
        self._nodes = api.Nodes(**self.requester_settings)
        self._operator = api.Operator(**self.requester_settings)
        self._regions = api.Regions(**self.requester_settings)
        self._sentinel = api.Sentinel(**self.requester_settings)
        self._status = api.Status(**self.requester_settings)
        self._system = api.System(**self.requester_settings)
        self._validate = api.Validate(**self.requester_settings)

    def get_uri(self):
        if self.secure:
            protocol = "https"
        else:
            protocol = "http"
        return "{protocol}://{host}".format(protocol=protocol, host=self.host)

    def get_namespace(self):
        return self.__namespace

    def get_token(self):
        return self.token

    @property
    def jobs(self):
        return self._jobs

    @property
    def job(self):
        return self._job

    @property
    def nodes(self):
        return self._nodes

    @property
    def node(self):
        return self._node

    @property
    def allocations(self):
        return self._allocations

    @property
    def allocation(self):
        return self._allocation

    @property
    def evaluations(self):
        return self._evaluations

    @property
    def evaluation(self):
        return self._evaluation

    @property
    def event(self):
        return self._event

    @property
    def agent(self):
        return self._agent

    @property
    def client(self):
        return self._client

    @property
    def deployments(self):
        return self._deployments

    @property
    def deployment(self):
        return self._deployment

    @property
    def regions(self):
        return self._regions

    @property
    def status(self):
        return self._status

    @property
    def system(self):
        return self._system

    @property
    def operator(self):
        return self._operator

    @property
    def validate(self):
        return self._validate

    @property
    def namespaces(self):
        return self._namespaces

    @property
    def namespace(self):
        return self._namespace

    @property
    def acl(self):
        return self._acl

    @property
    def sentinel(self):
        return self._sentinel

    @property
    def metrics(self):
        return self._metrics



from nomad.api.base import Requester


class Metrics(Requester):

    """
    The /metrics endpoint returns metrics for the current Nomad process.

    https://www.nomadproject.io/api/metrics.html

    Key metrics delivered via the endpoint could be found in Telemetry section:

    https://www.nomadproject.io/docs/agent/telemetry.html
    """
    ENDPOINT = "metrics"

    def __init__(self, **kwargs):
        super(Metrics, self).__init__(**kwargs)

    def __str__(self):
        return "{0}".format(self.__dict__)

    def __repr__(self):
        return "{0}".format(self.__dict__)

    def __getattr__(self, item):
        raise AttributeError

    def get_metrics(self):
        """ Get the metrics

           https://www.nomadproject.io/api/metrics.html

            returns: dict
            raises:
              - nomad.api.exceptions.BaseNomadException
              - nomad.api.exceptions.URLNotFoundNomadException
        """
        return self.request(method="get").json()
